<?xml version='1.0'?>

<xsl:stylesheet xmlns:xsl      = 'http://www.w3.org/1999/XSL/Transform'
                xmlns:xs       = 'http://www.w3.org/2001/XMLSchema'
                xmlns:wsdl     = 'http://schemas.xmlsoap.org/wsdl/'
                xmlns:lxslt    = 'http://xml.apache.org/xslt'
                xmlns:java     = 'http://xml.apache.org/xslt/java'
                xmlns:redirect = 'xalan://org.apache.xalan.lib.Redirect'
                xmlns:wf       = 'xalan://com.ibm.workflow.util.fdl.make'
                extension-element-prefixes='redirect'
                version='1.0'>

<xsl:include href='xsl:/common.xsl'/>

<xsl:param name='generator'/>
<xsl:param name='spackage'/>
<xsl:param name='basedir'/>
<xsl:param name='output'/>
<xsl:param name='sns'/>

<xsl:output method='text'/>
<xsl:template match='text()'/>

<xsl:template match='/'>
 <xsl:apply-templates/>
</xsl:template>

<xsl:template match='xs:complexType'>
 <xsl:choose>
  <xsl:when test='@name="nullType"'/>
  <xsl:when test='@name="EnquiryResult"'/>
  <xsl:when test='@name="BINARY_ARRAY"'/>
  <xsl:when test='@name="FLOAT_ARRAY"'/>
  <xsl:when test='@name="LONG_ARRAY"'/>
  <xsl:when test='@name="STRING_ARRAY"'/>
  <xsl:otherwise>
  <xsl:variable name='typeName' select='wf:WSDL.globalName(@name)'/>
  <xsl:variable name='fileName' select='wf:WSDL.getFileName($typeName, $basedir, $spackage, ".java")'/>
  <redirect:write select='$fileName'>
   <xsl:text>// MQSeries Workflow SOAP Wrapper class for '</xsl:text>
   <xsl:value-of select='$typeName'/>
   <xsl:text>' STRUCTUREs</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>// This file ("</xsl:text>
   <xsl:variable name='javaFile' select='java:java.io.File.new($fileName)'/>
   <xsl:variable name='absoluteFileName' select='java:getAbsolutePath($javaFile)'/>
   <xsl:value-of select='$absoluteFileName'/>
   <xsl:text>")</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>// was generated by </xsl:text>
   <xsl:value-of select='$generator'/>
   <xsl:text> on </xsl:text>
   <xsl:value-of select='java:java.util.Date.new()'/>
   <xsl:value-of select='$eol'/>
   <xsl:text>// from WSDL file "</xsl:text>
   <xsl:value-of select='$output'/>
   <xsl:text>"</xsl:text>
   <xsl:if test='$spackage!=""'>
    <xsl:value-of select='$eol'/>
    <xsl:value-of select='$eol'/>
    <xsl:text>package </xsl:text>
    <xsl:value-of select='$spackage'/>
    <xsl:text>;</xsl:text>
   </xsl:if>
   <xsl:value-of select='$eol'/>
   <xsl:value-of select='$eol'/>
   <xsl:text>import java.io.*;</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>import org.w3c.dom.*;</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>import org.apache.soap.*;</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>import org.apache.soap.encoding.soapenc.*;</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>import org.apache.soap.rpc.*;</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>import org.apache.soap.util.*;</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>import org.apache.soap.util.xml.*;</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>import com.ibm.workflow.soap.provider.ArrayDeserializer;</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:value-of select='$eol'/>
   <xsl:text>public class </xsl:text>
   <xsl:value-of select='$typeName'/>
   <xsl:value-of select='$eol'/>
   <xsl:text>{</xsl:text>
   <xsl:for-each select='.//xs:element'>
    <xsl:variable name='javaTypeName' select='wf:WSDL.getJavaType($spackage, @type, @maxOccurs)'/>
    <xsl:value-of select='$eol'/>
    <xsl:text>  public </xsl:text>
    <xsl:value-of select='$javaTypeName'/>
    <xsl:text> </xsl:text>
    <xsl:value-of select='@name'/>
    <xsl:text>;</xsl:text>
   </xsl:for-each>
   <xsl:value-of select='$eol'/>
   <xsl:value-of select='$eol'/>
   <xsl:text>  public </xsl:text>
   <xsl:value-of select='$typeName'/>
   <xsl:text>() { }</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:value-of select='$eol'/>
   <xsl:text>  public </xsl:text>
   <xsl:value-of select='$typeName'/>
   <xsl:text>(</xsl:text>
   <xsl:for-each select='.//xs:element'>
    <xsl:variable name='javaTypeName' select='wf:WSDL.getJavaType($spackage, @type, @maxOccurs)'/>
    <xsl:value-of select='$javaTypeName'/>
    <xsl:text> </xsl:text>
    <xsl:value-of select='@name'/>
    <xsl:if test='not(position()=last())'>
     <xsl:text>, </xsl:text>
    </xsl:if>
   </xsl:for-each>
   <xsl:text>)</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>  {</xsl:text>
   <xsl:for-each select='.//xs:element'>
    <xsl:value-of select='$eol'/>
    <xsl:text>    this.</xsl:text>
    <xsl:value-of select='@name'/>
    <xsl:text> = </xsl:text>
    <xsl:value-of select='@name'/>
    <xsl:text>;</xsl:text>
   </xsl:for-each>
   <xsl:value-of select='$eol'/>
   <xsl:text>  }</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:value-of select='$eol'/>
   <xsl:text>  public static class Deserializer implements Serializer, org.apache.soap.util.xml.Deserializer</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>  {</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>    public void marshall(String inSES, Class javaType, Object src, Object context, Writer sink,</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>                         NSStack nsStack, XMLJavaMappingRegistry xjmr, SOAPContext ctx)</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>      throws IllegalArgumentException, IOException</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>    {</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>      nsStack.pushScope();</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:value-of select='$eol'/>
   <xsl:text>      SoapEncUtils.generateStructureHeader(inSES, javaType, context, sink, nsStack, xjmr);</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:value-of select='$eol'/>
   <xsl:text>      sink.write(StringUtils.lineSeparator);</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:value-of select='$eol'/>
   <xsl:text>      </xsl:text>
   <xsl:value-of select='$spackage'/>
   <xsl:text>.</xsl:text>
   <xsl:value-of select='$typeName'/>
   <xsl:text> src2 = (</xsl:text>
   <xsl:value-of select='$spackage'/>
   <xsl:text>.</xsl:text>
   <xsl:value-of select='$typeName'/>
   <xsl:text>)src;</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:value-of select='$eol'/>
   <xsl:text>      Parameter param;</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:for-each select='.//xs:element'>
    <xsl:call-template name='serializeElement'/>
   </xsl:for-each>
   <xsl:value-of select='$eol'/>
   <xsl:text>      sink.write("&lt;/" + context + '&gt;');</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:value-of select='$eol'/>
   <xsl:text>      nsStack.popScope();</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>    }</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:value-of select='$eol'/>
   <xsl:text>    public Bean unmarshall(String inSES, QName elType, Node src, XMLJavaMappingRegistry xjmr, SOAPContext ctx)</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>      throws IllegalArgumentException</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>    {</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>      Element e = DOMUtils.getFirstChildElement((Element)src);</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:value-of select='$eol'/>
   <xsl:text>      </xsl:text>
   <xsl:value-of select='$spackage'/>
   <xsl:text>.</xsl:text>
   <xsl:value-of select='$typeName'/>
   <xsl:text> target;</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:value-of select='$eol'/>
   <xsl:text>      try</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>      {</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>        target = (</xsl:text>
   <xsl:value-of select='$spackage'/>
   <xsl:text>.</xsl:text>
   <xsl:value-of select='$typeName'/>
   <xsl:text>)</xsl:text>
   <xsl:value-of select='$spackage'/>
   <xsl:text>.</xsl:text>
   <xsl:value-of select='$typeName'/>
   <xsl:text>.class.newInstance();</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>      }</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>      catch (Exception xcpt) { throw new IllegalArgumentException("Problem instantiating bean: " + xcpt.getMessage()); }</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:value-of select='$eol'/>
   <xsl:text>      while (e != null)</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>      {</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>        String tagName = e.getTagName();</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:value-of select='$eol'/>
   <xsl:for-each select='.//xs:element'>
    <xsl:call-template name='deserializeElement'/>
   </xsl:for-each>
   <xsl:value-of select='$eol'/>
   <xsl:text>        e = DOMUtils.getNextSiblingElement(e);</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>      }</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:value-of select='$eol'/>
   <xsl:text>      return new Bean(</xsl:text>
   <xsl:value-of select='$spackage'/>
   <xsl:text>.</xsl:text>
   <xsl:value-of select='$typeName'/>
   <xsl:text>.class, target);</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>    }</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>  }</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:value-of select='$eol'/>
   <xsl:text>  private static final StringDeserializer stringDeserializer = new StringDeserializer();</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>  private static final LongDeserializer   longDeserializer   = new LongDeserializer  ();</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>  private static final DoubleDeserializer doubleDeserializer = new DoubleDeserializer();</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>  private static final Base64Serializer   binaryDeserializer = new Base64Serializer  ();</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:text>  private static final ArrayDeserializer  arrayDeserializer  = new ArrayDeserializer ();</xsl:text>
   <xsl:value-of select='$eol'/>
   <xsl:for-each select='wf:WSDL.getComplex(.//xs:element)'>
    <xsl:text>  private static final </xsl:text>
    <xsl:value-of select='.'/>
    <xsl:text>.Deserializer </xsl:text>
    <xsl:value-of select='.'/>
    <xsl:text>Deserializer = new </xsl:text>
    <xsl:value-of select='.'/>
    <xsl:text>.Deserializer();</xsl:text>
    <xsl:value-of select='$eol'/>
   </xsl:for-each>
   <xsl:text>}</xsl:text>
  </redirect:write>
 </xsl:otherwise>
 </xsl:choose>
</xsl:template>

<xsl:template name='serializeElement'>
 <xsl:value-of select='$eol'/>
 <xsl:text>      param = new Parameter("</xsl:text>
 <xsl:value-of select='@name'/>
 <xsl:text>", </xsl:text>
 <xsl:value-of select='wf:WSDL.getJavaType($spackage, @type, @maxOccurs)'/>
 <xsl:text>.class, </xsl:text>
 <xsl:value-of select='concat("src2.", @name)'/>
 <xsl:text>, null);</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:text>      xjmr.marshall(inSES, Parameter.class, param, null, sink, nsStack, ctx);</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:text>      sink.write(StringUtils.lineSeparator);</xsl:text>
 <xsl:value-of select='$eol'/>
</xsl:template>

<xsl:template name='deserializeElement'>
 <xsl:variable name='javaType'   select='wf:WSDL.getJavaType($spackage, @type, @maxOccurs)'/>
 <xsl:variable name='serializer' select='wf:WSDL.getSerializer(@type, @maxOccurs)'/>
 <xsl:text>        </xsl:text>
 <xsl:if test='position() != 1'>
  <xsl:text>else </xsl:text>
 </xsl:if>
 <xsl:text>if (tagName.equals("</xsl:text>
 <xsl:value-of select='@name'/>
 <xsl:text>"))</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:text>        {</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:text>          target.</xsl:text>
 <xsl:value-of select='@name'/>
 <xsl:text> = (</xsl:text>
 <xsl:value-of select='$javaType'/>
 <xsl:text>)</xsl:text>
 <xsl:value-of select='$serializer'/>
 <xsl:text>Deserializer.unmarshall(inSES, </xsl:text>
 <xsl:if test='$serializer = "array"'>
  <xsl:value-of select='wf:WSDL.getQName($spackage, @type)'/>
 </xsl:if>
 <xsl:if test='$serializer != "array"'>
  <xsl:text>elType</xsl:text>
 </xsl:if>
 <xsl:text>, e, xjmr, ctx).value;</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:if test='$serializer = "array"'>
  <xsl:value-of select='$eol'/>
  <xsl:text>          do { e = DOMUtils.getNextSiblingElement(e); } while (e != null &amp;&amp; e.getTagName().equals(tagName)); continue;</xsl:text>
  <xsl:value-of select='$eol'/>
 </xsl:if>
 <xsl:text>        }</xsl:text>
 <xsl:value-of select='$eol'/>
</xsl:template>

</xsl:stylesheet>
