<?xml version="1.0" ?>


<!-- ********************************************************* -->
<!-- *                                                       * -->
<!-- ********************************************************* -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:wf="http://www.ibm.com/workflow/2001"
                version="1.0">

<xsl:import href="xsl:/common.xsl"/>


<!-- ********************************************************* -->
<!-- *                                                       * -->
<!-- ********************************************************* -->

<xsl:template match="wf:person">
  <xsl:param name="procOption"/>
  <xsl:if test="$procOption='Create'">
    <xsl:text>CREATE&#x20;</xsl:text>
  </xsl:if>
  <xsl:if test="$procOption='Update'">
    <xsl:text>UPDATE&#x20;</xsl:text>
  </xsl:if>
  <xsl:if test="$procOption='Replace'">
    <xsl:text>REPLACE&#x20;</xsl:text>
  </xsl:if>
  <xsl:text>PERSON </xsl:text>
  <xsl:call-template name="singlequote">
    <xsl:with-param name="string" select="@userID"/>
  </xsl:call-template>
  <xsl:value-of select='$eol'/>
  <xsl:if test="wf:description">
    <xsl:apply-templates select="wf:description">
      <xsl:with-param name="ini" select="$indent"/>
    </xsl:apply-templates>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:if test="@lastName">
    <xsl:value-of select="$indent"/>
    <xsl:text>LAST_NAME </xsl:text>
    <xsl:call-template name="doublequote">
      <xsl:with-param name="string" select="@lastName"/>
    </xsl:call-template>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:if test="@firstName">
    <xsl:value-of select="$indent"/>
    <xsl:text>FIRST_NAME </xsl:text>
    <xsl:call-template name="doublequote">
      <xsl:with-param name="string" select="@firstName"/>
    </xsl:call-template>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:apply-templates select="wf:authorization">
    <xsl:with-param name="ini" select="$indent"/>
  </xsl:apply-templates>
  <xsl:if test="@absent='yes'">
    <xsl:value-of select="$indent"/>
    <xsl:text>IS_ABSENT </xsl:text>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:if test="@absent='no'">
    <xsl:value-of select="$indent"/>
    <xsl:text>IS_NOT_ABSENT </xsl:text>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:if test="@resetAbsent='yes'">
    <xsl:value-of select="$indent"/>
    <xsl:text>RESET_ABSENT </xsl:text>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:if test="@resetAbsent='no'">
    <xsl:value-of select="$indent"/>
    <xsl:text>DO NOT RESET_ABSENT </xsl:text>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:if test="@group">
    <xsl:value-of select="$indent"/>
    <xsl:text>GROUP </xsl:text>
    <xsl:call-template name="singlequote">
      <xsl:with-param name="string" select="@group"/>
    </xsl:call-template>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:if test="@system">
    <xsl:value-of select="$indent"/>
    <xsl:text>SYSTEM </xsl:text>
    <xsl:call-template name="singlequote">
      <xsl:with-param name="string" select="@system"/>
    </xsl:call-template>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:text>END </xsl:text>
  <xsl:call-template name="singlequote">
    <xsl:with-param name="string" select="@userID"/>
  </xsl:call-template>
  <xsl:value-of select='$eol'/>
  <xsl:value-of select='$eol'/>
</xsl:template>


<!-- ********************************************************* -->
<!-- * AUTHORIZATION                                         * -->
<!-- ********************************************************* -->

<xsl:template match="wf:authorization">
  <xsl:param name="ini"/>
  <xsl:value-of select="$indent"/>
  <xsl:text>AUTHORIZED_FOR </xsl:text>
  <xsl:choose>
    <xsl:when test="@type='authorization'">
      <xsl:text>AUTHORIZATION</xsl:text>
    </xsl:when>
    <xsl:when test="@type='operation'">
      <xsl:text>OPERATION</xsl:text>
    </xsl:when>
    <xsl:when test="@type='processModeling'">
      <xsl:text>PROCESS_MODELING</xsl:text>
    </xsl:when>
    <xsl:when test="@type='staff'">
      <xsl:text>STAFF</xsl:text>
    </xsl:when>
    <xsl:when test="@type='topology'">
      <xsl:text>TOPOLOGY</xsl:text>
    </xsl:when>
    <xsl:when test="@type='processCategory'">
      <xsl:text>PROCESS_CATEGORY </xsl:text>
      <xsl:choose>
        <xsl:when test="@allProcessCategories='yes'">
          <xsl:text>ALL</xsl:text>
        </xsl:when>
        <xsl:when test="@processCategory">
          <xsl:call-template name="singlequote">
            <xsl:with-param name="string" select="@processCategory"/>
          </xsl:call-template>
        </xsl:when>
      </xsl:choose>
      <xsl:if test="@isAdmin='yes'">
        <xsl:text> AS_ADMINISTRATOR</xsl:text>
      </xsl:if>
    </xsl:when>
    <xsl:when test="@type='workItems'">
      <xsl:text>WORKITEMS_OF </xsl:text>
      <xsl:choose>
        <xsl:when test="@allWorkItems='yes'">
          <xsl:text>ALL</xsl:text>
        </xsl:when>
        <xsl:when test="@owner">
          <xsl:call-template name="singlequote">
            <xsl:with-param name="string" select="@owner"/>
          </xsl:call-template>
        </xsl:when>
      </xsl:choose>
    </xsl:when>
    <xsl:when test="@type='workList'">
      <xsl:value-of select="$subitem"/>
      <xsl:text>WORKLIST_OF </xsl:text>
      <xsl:choose>
        <xsl:when test="@allWorkLists='yes'">
          <xsl:text>ALL</xsl:text>
        </xsl:when>
        <xsl:when test="@owner">
          <xsl:call-template name="singlequote">
            <xsl:with-param name="string" select="@owner"/>
          </xsl:call-template>
        </xsl:when>
      </xsl:choose>
    </xsl:when>
  </xsl:choose>
  <xsl:value-of select='$eol'/>
</xsl:template>

</xsl:stylesheet>
