<?xml version="1.0" ?>


<!-- ********************************************************* -->
<!-- *                                                       * -->
<!-- ********************************************************* -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xsd="http://www.w3.org/1999/XMLSchema"
                xmlns:wsfl="http://schemas.xmlsoap.org/wsfl"
                xmlns:wsdl="http://schemas.xmlsoap.org/wsdl"
                xmlns:wf="http://www.ibm.com/workflow/2001"
                xmlns:wfds="http://www.ibm.com/workflow/2001/dataStructure"
                version="1.0">

<xsl:import href="xsl:/common.xsl"/>


<!-- ********************************************************* -->
<!-- * STRUCTURE (wf:complexType)                            * -->
<!-- *   Param: ini - initial Indent (required for layout)   * -->
<!-- ********************************************************* -->

<xsl:template match="xsd:complexType">
  <xsl:param name="ini"/>
  <xsl:param name="procOption"/>
  <xsl:apply-templates
                  select="xsd:element[not(@type=preceding::xsd:element/@type)]"
                  mode="emit-elt-struct"/>
  <xsl:call-template name="to-struct">
    <xsl:with-param name="ini" select="$ini"/>
    <xsl:with-param name="procOption" select="$procOption"/>
    <xsl:with-param name="st-name" select="@name"/>
    <xsl:with-param name="st-memb" select="xsd:element"/>
  </xsl:call-template>
</xsl:template>

<xsl:template name="to-struct">
  <xsl:param name="ini"/>
  <xsl:param name="procOption"/>
  <xsl:param name="st-name"/>
  <xsl:param name="st-memb"/>
  <xsl:if test="$procOption='Create'">
    <xsl:text>CREATE&#x20;</xsl:text>
  </xsl:if>
  <xsl:if test="$procOption='Update'">
    <xsl:text>UPDATE&#x20;</xsl:text>
  </xsl:if>
  <xsl:if test="$procOption='Replace'">
    <xsl:text>REPLACE&#x20;</xsl:text>
  </xsl:if>
  <xsl:text>STRUCTURE&#x20;</xsl:text>
  <xsl:variable name="nstring">
    <xsl:call-template name="removePrefix">
      <xsl:with-param name="qname" select="$st-name"/>
    </xsl:call-template>
  </xsl:variable>
  <xsl:call-template name="singlequote">
    <xsl:with-param name="string" select="$nstring"/>
  </xsl:call-template>
  <xsl:value-of select='$eol'/>
  <xsl:apply-templates select="$st-memb">
    <xsl:with-param name="ini" select="$indent1"/>
  </xsl:apply-templates>
  <xsl:text>END&#x20;</xsl:text>
  <xsl:call-template name="singlequote">
    <xsl:with-param name="string" select="$nstring"/>
  </xsl:call-template>
  <xsl:value-of select='$eol'/>
  <xsl:value-of select='$eol'/>
</xsl:template>


<!-- ********************************************************* -->
<!-- * STRUCTURE (MemberElement): xsd:element                * -->
<!-- *   Param: ini - initial Indent (required for layout)   * -->
<!-- ********************************************************* -->

<xsl:template match="xsd:element">
  <xsl:param name="ini"/>
  <xsl:call-template name="member-element">
    <xsl:with-param name="ini" select="$ini"/>
    <xsl:with-param name="me-name" select="@name"/>
    <xsl:with-param name="me-type" select="@type"/>
  </xsl:call-template>
</xsl:template>

<xsl:template match="xsd:element" mode="emit-elt-struct">
  <xsl:variable name="type-lname">
    <xsl:call-template name="removePrefix">
      <xsl:with-param name="qname" select="@type"/>
    </xsl:call-template>
  </xsl:variable>
  <xsl:if test="not(($type-lname='string') or ($type-lname='long') or
                    ($type-lname='float') or ($type-lname='binary'))">
    <xsl:call-template name="emit-struct-for-type">
      <xsl:with-param name="type-name" select="$type-lname"/>
    </xsl:call-template>
  </xsl:if>
</xsl:template>

<xsl:template name="member-element">
  <xsl:param name="ini"/>
  <xsl:param name="me-name"/>
  <xsl:param name="me-type"/>
  <xsl:value-of select="$ini"/>
  <xsl:call-template name="singlequote">
    <xsl:with-param name="string" select="$me-name"/>
  </xsl:call-template>
  <xsl:text> : </xsl:text>
  <xsl:variable name="ltype">
    <xsl:call-template name="removePrefix">
      <xsl:with-param name="qname" select="$me-type"/>
    </xsl:call-template>
  </xsl:variable>
  <xsl:choose>
    <xsl:when test="$ltype='string'">
      <xsl:text>STRING</xsl:text>
    </xsl:when>
    <xsl:when test="$ltype='long'">
      <xsl:text>LONG</xsl:text>
    </xsl:when>
    <xsl:when test="$ltype='float'">
      <xsl:text>FLOAT</xsl:text>
    </xsl:when>
    <xsl:when test="$ltype='binary'">
      <xsl:text>BINARY</xsl:text>
    </xsl:when>
    <xsl:otherwise>
      <xsl:call-template name="singlequote">
        <xsl:with-param name="string" select="$ltype"/>
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
  <xsl:if test="@cardinality">
    <xsl:call-template name="parenthesize">
      <xsl:with-param name="string" select="string(@cardinality)"/>
    </xsl:call-template>
  </xsl:if>
  <xsl:text>;</xsl:text>
  <xsl:value-of select='$eol'/>
</xsl:template>

<!-- ********************************************************* -->
<!-- * STRUCTURE (wf:complexType)                            * -->
<!-- *   Param: ini - initial Indent (required for layout)   * -->
<!-- ********************************************************* -->


<xsl:template match="wsdl:message">
  <xsl:apply-templates
                select="wsdl:part[not(@element=preceding::wsdl:part/@element)]"
                mode="emit-part-struct"/>
  <xsl:call-template name="to-struct">
    <xsl:with-param name="st-name" select="@name"/>
    <xsl:with-param name="st-memb" select="wsdl:part"/>
  </xsl:call-template>
</xsl:template>

<xsl:template match="wsdl:part" mode="emit-part-struct">
  <xsl:call-template name="emit-struct-for-type">
    <xsl:with-param name="type-name" select="@element"/>
  </xsl:call-template>
</xsl:template>

<xsl:template name="emit-struct-for-type">
  <xsl:param name="type-name"/>
  <xsl:variable name="type-lname">
    <xsl:call-template name="removePrefix">
      <xsl:with-param name="qname" select="$type-name"/>
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="c-type"
          select="//xsd:schema/xsd:complexType[@name=$type-lname]"/>
  <xsl:choose>
    <xsl:when test="$c-type">
      <xsl:apply-templates select="$c-type"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:call-template name="emit-struct-from-import">
  <xsl:with-param name="imports" select="//wsdl:import"/>
  <xsl:with-param name="struct-name" select="$type-lname"/>
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template match="wsdl:part">
  <xsl:param name="ini"/>
  <xsl:variable name="prefix" select="substring-before(@element,':')"/>
  <xsl:variable name="ns-uri" select="string(name(namespace::*))"/>
  <xsl:call-template name="member-element">
    <xsl:with-param name="ini" select="$ini"/>
    <xsl:with-param name="me-name" select="@name"/>
    <xsl:with-param name="me-type" select="@element"/>
  </xsl:call-template>
</xsl:template>

<xsl:template name="emit-struct-from-import">
  <xsl:param name="imports"/>
  <xsl:param name="struct-name"/>
  <xsl:if test="$imports">
    <xsl:variable name="import" select="document($imports[1]/@location, /)"/>
    <xsl:variable name="c-type"
             select="$import//xsd:schema/xsd:complexType[@name=$struct-name]"/>
    <xsl:choose>
      <xsl:when test="$c-type">
        <xsl:apply-templates select="$c-type"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="emit-struct-from-import">
          <xsl:with-param name="imports" select="$imports[position() &gt; 1]"/>
          <xsl:with-param name="struct-name" select="$struct-name"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:if>
</xsl:template>

<!-- ********************************************************* -->
<!-- * removes the prefix of a qualified name                * -->
<!-- *   Param: qname - is a qualified name (prefix:name)    * -->
<!-- ********************************************************* -->

<xsl:template name="removePrefix">
  <xsl:param name="qname"/>
  <xsl:choose>
    <xsl:when test="contains($qname, ':')">
      <xsl:value-of select="substring-after($qname, ':')"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$qname"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

</xsl:stylesheet>
