// *****************************************************************************
// * IBM MQSeries Workflow XML Sample Scenario                                 *
// *****************************************************************************
//
//  Licensed Materials - Property of IBM
//  5697-FM3 5655-A96
//  (C) Copyright IBM Corp. 1999-2000
//  US Government Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract
//  with IBM Corp.
//
// *****************************************************************************
// *****************************************************************************
//  DISCLAIMER
//  ----------
//  This material contains programming source code for your consideration.
//  These examples have not been thoroughly tested under all conditions.
//  IBM, therefore, cannot guarantee or imply reliability, serviceability,
//  or function of these programs.
//  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
//  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
//  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
//  FOR A PARTICULAR PURPOSE.
// *****************************************************************************
package com.ibm.workflow.upes;

import org.w3c.dom.*;

/**
 * Some utility functions in order to simplify coding.
 * @version 1.0
 */
public class DOMUtils
{
  /**
   * Concat all the text and cdata node children of this elem and return
   * the resulting text.
   *
   * @param parentEl the element whose cdata/text node values are to
   *                 be combined.
   * @return the concatanated string.
   */
  static public String getChildCharacterData (Element parentEl)
  {
    if (parentEl == null)
    {
      return null;
    }
    Node          tempNode = parentEl.getFirstChild();
    StringBuffer  strBuf   = new StringBuffer();
    CharacterData charData;

    while (tempNode != null)
    {
      switch (tempNode.getNodeType())
      {
      case Node.TEXT_NODE :
      case Node.CDATA_SECTION_NODE : charData = (CharacterData)tempNode;
        strBuf.append(charData.getData());
        break;
      }
      tempNode = tempNode.getNextSibling();
    }
    return strBuf.toString();
  }  
  /**
   * Return the first child element of the given element. Null if no
   * children are found.
   *
   * @param elem Element whose child is to be returned
   * @return the first child element.
   */
  public static Element getFirstChildElement (Element elem)
  {
    for (Node n = elem.getFirstChild (); n != null; n = n.getNextSibling ())
    {
      if (n.getNodeType () == Node.ELEMENT_NODE)
      {
        return(Element) n;
      }
    }
    return null;
  }  

  /**
   * Return the first child element with a given name of the given element. 
   * Null if no children are found.
   * 
   * @param elem   Element whose child is to be returned
   * @return the first child element.
   */
  public static Element getElementByTagName(Element elem, String tagName) 
  {
    NodeList elementList = elem.getElementsByTagName( tagName);
    if ( elementList.getLength() > 0)
    {
      return (Element)elementList.item(0);
    }
    return null;
  }

  /**
   * Return the next sibling element of the given element. Null if no
   * more sibling elements are found.
   *
   * @param elem Element whose sibling element is to be returned
   * @return the next sibling element.
   */
  public static Element getNextSiblingElement (Element elem) {
    for (Node n = elem.getNextSibling (); n != null; n = n.getNextSibling ())
    {
      if (n.getNodeType () == Node.ELEMENT_NODE)
      {
        return(Element) n;
      }
    }
    return null;
  }  
}
