// *****************************************************************************
// * IBM MQSeries Workflow XML Sample Scenario                                 *
// *****************************************************************************
//
//  Licensed Materials - Property of IBM
//  5697-FM3 5655-A96
//  (C) Copyright IBM Corp. 1999-2000
//  US Government Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract
//  with IBM Corp.
//
// *****************************************************************************
// *****************************************************************************
//  DISCLAIMER
//  ----------
//  This material contains programming source code for your consideration.
//  These examples have not been thoroughly tested under all conditions.
//  IBM, therefore, cannot guarantee or imply reliability, serviceability,
//  or function of these programs.
//  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
//  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
//  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
//  FOR A PARTICULAR PURPOSE.
// *****************************************************************************
package com.ibm.workflow.upes;

import org.w3c.dom.NodeList;

import org.w3c.dom.Element;

/**
 * The <code>InvokeProgramData</code> class represents
 * the InvokeProgram XML message from MQSeries Workflow
 * as a Java class.
 * 
 * For each possible entry in the XML message there is a data memeber of this class.
 * <p>
 * It has the ability to traverse a given DOM Tree (i.e. a given rootElement of type Element)
 * and to fill out all given information during the traversal.
 * <p>
 * @version 1.0
 */
public class InvokeProgramData
{
/*********************************************************************************
 * <b>_ResponseQueueName</b> is the queue name of the queue to which a response may be sent.
 **********************************************************************************/
  public String                 _ResponseQueueName;
/*********************************************************************************
 * <b>_ResponseQueueManagerName</b> is the queue manager name of the queue
 * to which a response may be sent.
 **********************************************************************************/
  public String                 _ResponseQueueManagerName;
/*********************************************************************************
 * <b>_ResponseRequired</b> contains whether a response is required.
 **********************************************************************************/
  public boolean                _ResponseRequired;
/*********************************************************************************
 * <b>_UserContext</b> contains the user context given with the message header.
 **********************************************************************************/
  public String                 _UserContext;
  public String                 _ActImplCorrelID;
  public String                 _Starter;
  public String                 _ProgramName;
  public String                 _ProcTemplID;
  public String                 _ProgramParameters;
  public ImplementationData[]   _ImplData;
  public Container              _ProgramInputData;
  public Container              _ProgramOutputDataDefaults;


/************************************************************
 * <b>InvokeProgramData</b> the constructor of the class, which parses the given root element 
 * of the DOM tree which represents the message.
 *
 * @parameters <b>rootElement</b> the root element of the DOM tree which represents the 
 * message.
 * @parameters <b>responseQ</b> the name of the response queue to which response is to be put.
 * @parameters <b>responseQMgr</b> the name of the response queue manager to which 
 * the response is to be sent.
 * 
 ***********************************************************/
  public InvokeProgramData(Element  rootElement,String responseQ, String responseQMgr)
  {
    try
    {
      _ResponseRequired = true;
      _ResponseQueueName = responseQ;
      _ResponseQueueManagerName = responseQMgr;

      Element  headerElement = DOMUtils.getElementByTagName( rootElement,"WfHeader");
      //Process header information to retrieve response behaviour/data
      if (headerElement == null)
      {
        Tracer.trace("No header information ---> ResponseRequired is assumed to true");
      }
      else
      {
        Element  RRElement = DOMUtils.getElementByTagName( headerElement,"ResponseRequired");
        if (RRElement != null)
        {
          String strRR = DOMUtils.getChildCharacterData( RRElement);
          _ResponseRequired = WfXMLValueConverter.FromXMLBool( strRR);
        }
        Element  UCElement = DOMUtils.getElementByTagName( headerElement,"UserContext");
        if (UCElement != null)
        {
          _UserContext = DOMUtils.getChildCharacterData( UCElement);
        }
      }
      //Process WfMessage itself
      Element invokeImplElement = DOMUtils.getElementByTagName( rootElement,"ActivityImplInvoke");
      if (invokeImplElement != null)
      {
        Element aICIDElement = DOMUtils.getElementByTagName( invokeImplElement,"ActImplCorrelID");
        if (aICIDElement != null)
        {
          _ActImplCorrelID = DOMUtils.getChildCharacterData( aICIDElement);
        }
        Element starterElement = DOMUtils.getElementByTagName( invokeImplElement,"Starter");
        if ( starterElement != null)
        {
          _Starter = DOMUtils.getChildCharacterData( starterElement);
        }
        ExtractMainData(invokeImplElement);
      }
    }
    catch (Exception e)
    {
      Tracer.trace("error: "+e.getMessage());
      e.printStackTrace();
    }
  }  
/************************************************************
 * <b>ExtractMainData</b> extracts the main invoke program data like implementation data 
 * program name etc. out of the DOM subtree.
 * For each implementation data a new entry is appended into _ImplData.
 * @parameters <b>ActInvokeElement</b> the root DOM element representing the implementation data
 * 
 ***********************************************************/
  private void ExtractMainData(Element ActInvokeElement)
  {
    UPESListener l = null;

    try
    {
      //first get general information
      Element programIDElement = DOMUtils.getElementByTagName( ActInvokeElement,"ProgramID");
      if (programIDElement != null)
      {
        Element elProgramName = DOMUtils.getElementByTagName( programIDElement,"ProgramName");
        if ( elProgramName != null)
        {
          _ProgramName = DOMUtils.getChildCharacterData( elProgramName);
        }
        Element elProcTemplID = DOMUtils.getElementByTagName( programIDElement,"ProcTemplID");
        if ( elProcTemplID != null)
        {
          _ProcTemplID = DOMUtils.getChildCharacterData( elProcTemplID);
        }
      }
      //for ImplementationData nodes in the tree generate an ImplementationData object
      //and put it into the array.
      NodeList inputDataNodeList = ActInvokeElement.getElementsByTagName( "ImplementationData");
      int implDataCount = inputDataNodeList.getLength();
      _ImplData = new ImplementationData[implDataCount];
      for (int i = 0; i < implDataCount; i++)
      {
        _ImplData[i] = new ImplementationData((Element)inputDataNodeList.item(i));
      }
      //now extract the input container
      Element inputCntrElement = DOMUtils.getElementByTagName( ActInvokeElement,"ProgramInputData");
      if (inputCntrElement != null)
      {
        _ProgramInputData = new Container(inputCntrElement);
      }
      //extract the default output container
      Element outputDefaultsCntrElement = DOMUtils.getElementByTagName( ActInvokeElement,"ProgramOutputDataDefaults");
      if (outputDefaultsCntrElement != null)
      {
        _ProgramOutputDataDefaults = new Container(outputDefaultsCntrElement);
      }
      //now extract the program parametersr
      Element programParameters = DOMUtils.getElementByTagName( ActInvokeElement, "ProgramParameters");
      if (programParameters != null)
      {
        _ProgramParameters = DOMUtils.getChildCharacterData( programParameters);
      }
    }
    catch (Exception e)
    {
      Tracer.trace("error: "+e.getMessage());
      e.printStackTrace();
    }
  }  
}      
