// *****************************************************************************
// * IBM MQSeries Workflow XML Sample Scenario                                 *
// *****************************************************************************
//
//  Licensed Materials - Property of IBM
//  5697-FM3 5655-A96
//  (C) Copyright IBM Corp. 1999-2000
//  US Government Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract
//  with IBM Corp.
//
// *****************************************************************************
// *****************************************************************************
//  DISCLAIMER
//  ----------
//  This material contains programming source code for your consideration.
//  These examples have not been thoroughly tested under all conditions.
//  IBM, therefore, cannot guarantee or imply reliability, serviceability,
//  or function of these programs.
//  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
//  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
//  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
//  FOR A PARTICULAR PURPOSE.
// *****************************************************************************
package com.ibm.workflow.upes;

import java.lang.*;

/**
 * The <code>InvokeProgramResponseData</code> class represents
 * the InvokeProgramResponse XML message from MQSeries Workflow
 * as a Java class.
 * 
 * 
 * 
 * 
 * program execution server. This means that it reads messages from
 * a specified input queue and supports (registered) UPESListeners
 * with the recieved message.
 * <p>
 * An UPES listens to one input queue which must be specified with
 * queue name and queue manager name.
 * It is able to send messages to an arbitrary queue also specified
 * by queue and queue manager name.
 * <p>
 * UPESListener can register or deregister to the UPES so that when
 * a message arrives the UPES asks each listener if it is responsible
 * to handle the message. It then gives the message to the first listerer
 * which claims the responsibility.
 * <p>
 * @version 1.0
 */
public class InvokeProgramResponseData
{
  private String    _ActImplCorrelID;
  private String    _ProgramRC;
  private Container _ProgramOutputData;
  private String    _ExceptionMessageText;

/************************************************************
 * <b>InvokeProgramResponseData</b> constructor for the 'good' case (i.e. non-exception case)
 * @parameters <b>ActImplCorrelID</b> the activity correlation id which is to be set in the response message.
 * @parameters <b>ProgramRC</b> the return code is to be set in the response message.
 * @parameters <b>ProgramOutputData</b> the output container which is to be set in the response message.
 *
 * @see Container
 ***********************************************************/
  public InvokeProgramResponseData(String ActImplCorrelID,String ProgramRC,Container ProgramOutputData)
  {
    _ActImplCorrelID   = ActImplCorrelID;
    _ProgramRC         = ProgramRC;
    _ProgramOutputData = ProgramOutputData;
  } 
/************************************************************
 * <b>InvokeProgramResponseData</b> constructor for the 'bad' case (i.e. exception case)
 * @parameters <b>ActImplCorrelID</b> the activity correlation id which is to be set in the response message.
 * @parameters <b>ProgramRC</b> the return code is to be set in the response message.
 * @parameters <b>ExceptionMessageText</b> the message text which is to be set in the response message.
 *
 * @see Container
 ***********************************************************/

  public InvokeProgramResponseData(String ActImplCorrelID,String ProgramRC,String ExceptionMessageText)
  {
    _ActImplCorrelID   = ActImplCorrelID;
    _ProgramRC         = ProgramRC;
    _ExceptionMessageText = ExceptionMessageText;
  } 
/************************************************************
 * <b>GenerateXMLResponse</b> generates the XML response message out of the given 
 * data members.
 * @return The response message as XML string 
 * 
 ***********************************************************/
  public String GenerateXMLResponse()
  {
    String ResponseMessage;
    if (_ExceptionMessageText == null)
    {
      if (_ActImplCorrelID == null)
      {
        _ActImplCorrelID = "";
      }
      if (_ProgramRC == null)
      {
        _ProgramRC = "0";
      }
      ResponseMessage =
      "<?xml version=\"1.0\" standalone=\"yes\"?>"
      +""
      +"<WfMessage>" 
      +  "  <WfMessageHeader>" + "\n" 
      +  "    <ResponseRequired>" + "Yes" 
      +  "    </ResponseRequired>" + "\n" 
      +  "  </WfMessageHeader>" + "\n" 
      +"  <ActivityImplInvokeResponse>"
      +"    <ActImplCorrelID>"+_ActImplCorrelID+"</ActImplCorrelID>"
      +"    <ProgramRC>"+_ProgramRC+"</ProgramRC>"
      +"    <ProgramOutputData>";
      if (_ProgramOutputData != null)
      {
        ResponseMessage = ResponseMessage +"      "+_ProgramOutputData.AsXMLString();
      }
      ResponseMessage = ResponseMessage +"    </ProgramOutputData>"
                        +"  </ActivityImplInvokeResponse>"
                        +"</WfMessage>";
    }
    else
    {
      ResponseMessage =
      "<?xml version=\"1.0\" standalone=\"yes\"?>"
      +""
      +"<WfMessage>"
      +  "  <WfMessageHeader>" + "\n" 
      +  "    <ResponseRequired>" + "Yes" 
      +  "    </ResponseRequired>" + "\n" 
      +  "  </WfMessageHeader>" + "\n" 
      +"  <ActivityImplInvokeResponse>"
      +"    <ActImplCorrelID>"+_ActImplCorrelID+"</ActImplCorrelID>"
      +"    <Exception>"
      +"       <Rc>"+_ProgramRC+"</Rc>"
      +"       <MessageText>"
      +"      "+_ExceptionMessageText
      +"       </MessageText>"
      +"    </Exception>"
      +"  </ActivityImplInvokeResponse>"
      +"</WfMessage>";
    }
    return ResponseMessage; 
  }  
}      
