// *****************************************************************************
// * IBM MQSeries Workflow XML Sample Scenario                                 *
// *****************************************************************************
//
//  Licensed Materials - Property of IBM
//  5697-FM3 5655-A96
//  (C) Copyright IBM Corp. 1999-2000
//  US Government Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract
//  with IBM Corp.
//
// *****************************************************************************
// *****************************************************************************
//  DISCLAIMER
//  ----------
//  This material contains programming source code for your consideration.
//  These examples have not been thoroughly tested under all conditions.
//  IBM, therefore, cannot guarantee or imply reliability, serviceability,
//  or function of these programs.
//  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
//  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
//  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
//  FOR A PARTICULAR PURPOSE.
// *****************************************************************************
package com.ibm.workflow.sample.xml;

import com.ibm.workflow.upes.*;
import java.util.Date;

/**
 * The <b>CreditApprovalSampleListener</b> class is a sample implemenation of the UPESListener
 * interface. It accepts InvokeProgram messages when the program name is either 'RequestCreditApproval'
 * or 'CreditApproval'. It then brings up an <b>'ApprovalView'</b> with the given message information.
 * While this listener is processing a message the <b>_ready</b> field is set to false. This means
 * that <b>IsActive()</b> will return false in order to indicate that the listener is busy.
 * This class supports a log to which entries are made each time somthing is processed.
 * @version 1.0
 */
class CreditApprovalSampleListener implements UPESListener
{
    /*****************************************************************************
     * While the listener is processing a message <b>_u</b> contains the UPES from which the call came.
     * There are no accessors for this datamember.
     * @see UPES
     *
    *****************************************************************************/
    protected transient java.util.Vector _aChangeListener = null;
    protected transient java.beans.PropertyChangeSupport _propertyChange;
    /*****************************************************************************
     * <b>_fieldLogListing</b> contains the log entries which where generated during the livetime of the object.
     * @see #AppendLogEntry
     * @see #getLogListing
     * @see #setLogListing
     *
    *****************************************************************************/
    private javax.swing.DefaultListModel _fieldLogListing = new javax.swing.DefaultListModel();
    /*****************************************************************************
     * <b>_ready</b> is set to true if the listener is ready to receive and process messages.
     * This flag can be tested with the <b>IsActive</b> method.
     * @see #IsActive
     *
    *****************************************************************************/
    private boolean   _ready;
    private boolean   _asyncWait;
    private boolean   _approve;
    /*****************************************************************************
     * <b>CreditApprovalSampleListener</b> contructor of the listener.
     * 
     *****************************************************************************/
    public CreditApprovalSampleListener()
    {
        //setLogentries("");
        _ready = true;
        AppendLogEntry("CreditApprovalSampleListener started ");

    }  
    public synchronized void addPropertyChangeListener(java.beans.PropertyChangeListener listener)
    {
        getPropertyChange().addPropertyChangeListener(listener);
    }
    /*****************************************************************************
     * <b>AppendLogEntry</b> appends a log entry to the end of the log.
     * @parameters <b>append</b> the entry which is to be appended to the end
     * 
     *****************************************************************************/
    public void AppendLogEntry(String append)
    {
        getLogListing().addElement((new Date()).toString()+"   "+append);
        firePropertyChange("logListing", getLogListing(), getLogListing());
    }  
    public void firePropertyChange(String propertyName, Object oldValue, Object newValue)
    {
        getPropertyChange().firePropertyChange(propertyName, oldValue, newValue);
    }
    public javax.swing.DefaultListModel getLogListing()
    {
        return _fieldLogListing;
    }
    /*****************************************************************************
     * <b>getName</b> returns the Name of the listener and is hardcoded.
     * @return  returns "CreditApprovalSampleListener" the Name of the listener and is hardcoded.
     * 
     *****************************************************************************/
    public String getName()
    {
        return "CreditApprovalSampleListener";
    }  
    protected java.beans.PropertyChangeSupport getPropertyChange()
    {
        if(_propertyChange == null)
        {
            _propertyChange = new java.beans.PropertyChangeSupport(this);
        };
        return _propertyChange;
    }
    /*****************************************************************************
     * <b>IsActive</b> returns whether the listener is ready to receive and process incoming messages.
     * @return  <b>true</b>, if the listener is ready to receive and process incoming messages.
     * <p><b>false</b>, if the listener is <b>not</b> ready to receive and process incoming messages.
     * 
     *****************************************************************************/
    public boolean IsActive()
    {
        if(_ready)
        {
            AppendLogEntry("Activity signaled");
            return true;
        }
        else
        {
            AppendLogEntry("Not Readiness signaled (Listener is processing a message)");
            return false;
        }
    }  
    /*****************************************************************************
     * <b>IsResponsibleFor</b> returns whether the listener is responsible for a given message
     * It is responsible for message in which the program name is either 'RequestCreditApproval'
     * or 'CreditApproval'.
     *
     * @parameters  <b>msg</b> the message for which the listener is ask for responsibility.
     * @return  <b>true</b>, if the program name of msg is either 'RequestCreditApproval'
     * or 'CreditApproval'.
     *<p><b>false</b> otherwise
     * 
     *****************************************************************************/
    public boolean IsResponsibleFor(InvokeProgramData msg)
    {
        if(msg._ProgramName.equals("CreditApproval"))
        {
            AppendLogEntry("Responsibility for CreditApproval accepted");
            return true;
        }
        else if(msg._ProgramName.equals("RequestCreditApproval"))
        {
            AppendLogEntry("Responsibility for RequestCreditApproval accepted");
            return true;
        }
        else
        {
            AppendLogEntry("Responsibility denied");
            return false;
        }
    }  
    /*****************************************************************************
     * <b>ProcessMessage</b> processes the handed over message. During the processing _ready is set to
     * false in order to signal that the listener is busy. During the process a view will be brought up
     * (ApprovalView). The view when ready itself calls back the ProcessResponse method of the class
     * in order to send back the response message to the workflow system.
     *
     *
     * @parameters <b>u</b> the UPES from which the call came in.
     * @parameters <b>msg</b> message which is to be processed.
     *
     * @see UPES
     * @see InvokeProgramData
     *
     *****************************************************************************/
    public void ProcessMessage(UPES upes, MQHandler MQH, InvokeProgramData msg)
    {
        Tracer.trace("CreditApprovalSampleListener.ProcessMessage entry");
        String CustomerNo;
        String Account;
        String Amount;

        //set busy flag on  --> will be set back during ProcessResponse 
        _ready = false; 

        AppendLogEntry("Start Processing Message");
        AppendLogEntry("    Starter is: "+msg._Starter);
        ApprovalView apv;
        //Approval view will be brought up
        _asyncWait = true;
        apv = new ApprovalView(msg,this);
        apv.show();
        while(_asyncWait)
        {
            try
            {
                Thread.currentThread().sleep(100);
            }
            catch(Throwable t)
            {
            }
        }
        ProcessResponse(upes,msg,MQH);
        Tracer.trace("CreditApprovalSampleListener.ProcessMessage exit");
    }  
    /*****************************************************************************
    * <b>ProcessResponse</b> sends the response back to the UPES for a given message.
    * The input container from the input message is taken some information is put in and then it is 
    * used as output container for the response message. For this purpose the _ACTIVITY member must be
    * set to "".
    *
    *
    *****************************************************************************/
    public void ProcessResponse(UPES upes, InvokeProgramData msg, MQHandler MQH) 
    {
        try
        {
            Container OutputContainer = msg._ProgramInputData;
            ContainerElement AA;
            //check whether 'AddApproval' is already part of the container
            AA = OutputContainer.GetRootElement().GetChildNamed("CreditInfo").GetChildNamed("AddApproval");
            if(AA == null)
            {
                AA = new ContainerElement("AddApproval");
                OutputContainer.GetRootElement().GetChildNamed("CreditInfo").AddChild(AA);
            }
            if(_approve)
            {
                AA.SetValue("Y");
            }
            else
            {
                AA.SetValue("N");
            }
            ContainerElement ACT;
            //_ACTIVITY must be set to ""
            //          ACT = OutputContainer.GetRootElement().GetChildNamed("CreditInfo").GetChildNamed("_ACTIVITY");
            //          if (ACT == null)
            //             {
            //             ACT = new ContainerElement("_ACTIVITY");
            //             OutputContainer.GetRootElement().GetChildNamed("CreditInfo").AddChild(ACT);
            //             }
            //          ACT.SetValue("");
            //now the response data is created
            //_ACTIVITY must be delete
            ACT = OutputContainer.GetRootElement().GetChildNamed("CreditInfo").GetChildNamed("_ACTIVITY");
            if(ACT != null)
            {
                OutputContainer.GetRootElement().GetChildNamed("CreditInfo").RemoveChild(ACT);
            }
            InvokeProgramResponseData responseData 
                = new InvokeProgramResponseData(msg._ActImplCorrelID,"0",OutputContainer);
            upes.SendMessage(MQH, responseData,msg._ResponseQueueManagerName,msg._ResponseQueueName);
            //I am not busy anymore
            _ready = true;
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
        AppendLogEntry("End Processing Message");
    }
    public synchronized void removePropertyChangeListener(java.beans.PropertyChangeListener listener)
    {
        getPropertyChange().removePropertyChangeListener(listener);
    }
    public void setLogListing(javax.swing.DefaultListModel logListing)
    {
        javax.swing.DefaultListModel oldValue = _fieldLogListing;
        _fieldLogListing = logListing;
        firePropertyChange("logListing", oldValue, logListing);
    }
    public void TerminateAsyncWait()
    {
        _asyncWait = false;
    }
    public boolean getApprove()
    {
        return _approve;
    }
    public void setApprove(boolean approve)
    {
        _approve = approve;
    }
}
