/*BEGIN CMVC */
//*************************************************************************
//
//  Workfile: %W%
//  Last update: %E% %U%
//  SCCS path, id: %F% %I%
//
//************************************************************************/
/*END CMVC */
/*BEGIN COPYRIGHT */
//*************************************************************************
//
//  Licensed Materials - Property of IBM
//  WA07 Support Pac - Web Services Process Management Toolkit
//  (c) Copyright IBM Corp. 2001
//  US Government Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract
//  with IBM Corp.
//
//************************************************************************/
/*END COPYRIGHT */
/*BEGIN DISCLAIMER */
//*************************************************************************
//  DISCLAIMER
//  ----------
//  This material contains programming source code for your consideration.
//  These examples have not been thoroughly tested under all conditions.
//  IBM, therefore, cannot guarantee or imply reliability, serviceability,
//  or function of these programs.
//  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
//  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
//  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
//  FOR A PARTICULAR PURPOSE.
//************************************************************************/
/*END DISCLAIMER */

package com.ibm.workflow.util.fdl.make;

import java.util.*;
import java.io.*;

/**
 *
 * A FilterWriter which can be used to post-process the output of a visitor. It will look for %XXX% tags, and
 * if it finds a corresponding XXX=YYY entry in its map, it will replace the %XXX% tag with YYY.
 *
 * @author  Robert Junghuber (<a href='mailto:junghub@de.ibm.com'>junghub@de.ibm.com</a>)
 * @version 1.2
 *
 */

public final class SubstWriter extends FilterWriter
{
  /**
   *
   * Construct a SubstWriter on top of a given Writer.
   *
   */

  public SubstWriter(Writer out, Hashtable map) { super(out); this.map = map; }

  /**
   *
   * Write a single character
   *
   */

  public final void write(int c) throws IOException { buf.append(c); }

  /**
   *
   * Write a (sub-array of a) character array
   *
   */

  public final void write(char cbuf[], int off, int len) throws IOException { buf.append(cbuf, off, len); }

  /**
   *
   * Write a (sub-) string
   *
   */

  public final void write(String s, int off, int len) throws IOException { buf.append(s.substring(off, off + len)); }

  /**
   *
   * Overwrite flush with no-op.
   *
   */

  public void flush() { }

  /**
   *
   * Perform tag substitution, flush the buffer and close the stream.
   *
   */

  public void close() throws IOException
  {
    SubstFilter.format(buf, map);

    super.write(buf.toString(), 0, buf.length());
    super.flush();
    super.close();
  }

  private StringBuffer buf = new StringBuffer(10 * 1024);
  private Hashtable    map = null;
}
