/*BEGIN CMVC */
//*************************************************************************
//
//  Workfile: %W%
//  Last update: %E% %U%
//  SCCS path, id: %F% %I%
//
//************************************************************************/
/*END CMVC */
/*BEGIN COPYRIGHT */
//*************************************************************************
//
//  Licensed Materials - Property of IBM
//  WA07 Support Pac - Web Services Process Management Toolkit
//  (c) Copyright IBM Corp. 2001
//  US Government Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract
//  with IBM Corp.
//
//************************************************************************/
/*END COPYRIGHT */
/*BEGIN DISCLAIMER */
//*************************************************************************
//  DISCLAIMER
//  ----------
//  This material contains programming source code for your consideration.
//  These examples have not been thoroughly tested under all conditions.
//  IBM, therefore, cannot guarantee or imply reliability, serviceability,
//  or function of these programs.
//  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
//  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
//  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
//  FOR A PARTICULAR PURPOSE.
//************************************************************************/
/*END DISCLAIMER */

package com.ibm.workflow.util.fdl.make;

import java.io.*;
import java.util.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;
import org.apache.xpath.NodeSet;
import org.w3c.dom.*;
import org.xml.sax.*;
import com.ibm.workflow.util.fdl.*;
import com.ibm.workflow.util.wsdl.*;

/**
 *
 * Web Services Definition Generator. Generates WSDL, SOAP (de-)serializer-, proxy- and wrapper- sources and a SOAP
 * deployment descriptors scripts for MQSeries Workflow web services from a given FDL.
 *
 * @author  Robert Junghuber (<a href='mailto:junghub@de.ibm.com'>junghub@de.ibm.com</a>)
 * @version 1.2
 *
 */

public final class WSDL extends Make
{
  /**
   *
   * <p>An FDL parse tree visitor which collects the information about all FDL PROCESS'es's in the tree.
   * This data is then used by the code-generation method (<code>generateWSDL/code>) to generate WSDL and
   * related files.</p>
   *
   */

  public static class Visitor extends StructureVisitor
  {
    // Helper classes
    // --------------

    private class Process extends Setting // Represents an FDL process
    {
      public void toWSDLMessage(Hashtable once)
      {
        if (ddOptions == null)                    ddOptions = (Hashtable)cxt.getConfig().get(name);
        if (ddOptions == null)                    ddOptions = new Hashtable();
        if (!ddOptions.containsKey("ServiceUrn")) ddOptions.put("ServiceUrn", name);
        if (ddOptions.size() == 1)                ddOptions.put("call", name);

        for (Enumeration e = ddOptions.keys(); e.hasMoreElements(); )
        {
          String op = (String)e.nextElement();

          if      ("ServiceUrn".equals(op)) continue;
          else if ("call"      .equals(op)) msgDecl(op, input, output, once);
          else if ("spawn"     .equals(op)) msgDecl(op, input, null  , once);
          else if ("enquire"   .equals(op)) msgDecl(op, null , null  , once);
          else                              msgDecl(op, null , null  , once);
        }
      }

      private void msgDecl(String op, Structure in, Structure out, Hashtable once)
      {
        String msgName = name + op.substring(0, 1).toUpperCase() + op.substring(1);

        if (!once.containsKey(msgName))
        {
          String tIn  = in  == null || in .members.size() == 0 ? null : "%SNS%:" + in .name;
          String tOut = out == null || out.members.size() == 0 ? null : "%SNS%:" + out.name;

          if ("enquire".equals(op)) { tIn = null; tOut = "%WFNS%:EnquiryResult"; }

          msgBody(msgName, tIn,  true );
          msgBody(msgName, tOut, false);

          once.put(msgName, new Boolean(true));
        }
      }

      private void msgBody(String n, String t, boolean d)
      {
        pw.println("<wsdl:message name='" + n + "%" + (d ? "Request" : "Response") + "Postfix%'" + (t == null ? "/": "") + ">");

        if (t != null) pw.println(" <wsdl:part name='" + (d ? "In" : "Out") + "put' type='" + t + "'/>" + eol + "</wsdl:message>");

        pw.print(eol);
      }

      public void toWSDLPortType()
      {
        pw.println("<wsdl:portType name='" + name + "'>");

        for (Enumeration e = ddOptions.keys(); e.hasMoreElements(); )
        {
          String op  = (String)e.nextElement();
          String msg = name + op.substring(0, 1).toUpperCase() + op.substring(1);

          if ("ServiceUrn".equals(op)) continue;

          pw.println(" <wsdl:operation name='" + ddOptions.get(op) + "'>" + eol +
                     "  <wsdl:input  message='tns:"  + msg + "%RequestPostfix%'/>" + eol +
                     "  <wsdl:output message='tns:" + msg + "%ResponsePostfix%'/>" + eol +
                     " </wsdl:operation>");
        }

        pw.println("</wsdl:portType>" + eol);
      }

      public void toWSDLBinding()
      {
        pw.println("<wsdl:binding name='" + name + "%BindingPostfix%" + "' type='tns:" + name + "'>" + eol +
                   " <soap:binding style='rpc' transport='http://schemas.xmlsoap.org/soap/http'/>");

        for (Enumeration e = ddOptions.keys(); e.hasMoreElements(); )
        {
          String op = (String)e.nextElement();

          if ("ServiceUrn".equals(op)) continue;

          String urn = (String)ddOptions.get("ServiceUrn");

          pw.println(" <wsdl:operation name='" + ddOptions.get(op) + "'>" + eol +
                     "  <soap:operation soapAction='%ServiceNamespace%/" + name + "#" + op + "'/>" + eol +
                     "  <wsdl:input>"  + eol + soapBody(urn) + eol + "  </wsdl:input>"  + eol +
                     "  <wsdl:output>" + eol + soapBody(urn) + eol + "  </wsdl:output>" + eol +
                     " </wsdl:operation>");
        }

        pw.println("</wsdl:binding>" + eol);
      }

      private String soapBody(String urn)
      {
        return "   <soap:body use='encoded' encodingStyle='http://schemas.xmlsoap.org/soap/encoding/' namespace='urn:" + urn + "'/>";
      }

      public void toWSDLService()
      {
        pw.println(" <wsdl:port name='" + name + "%PortPostfix%" + "' binding='tns:" + name + "%BindingPostfix%" + "'>" + eol +
                   "  <soap:address location='%ServiceSoapLocation%'/>" + eol +
                   " </wsdl:port>" + eol);
      }

      public void toDeploymentDescriptor() throws IOException
      {
        String    svcUrn     = (String)ddOptions.get("ServiceUrn");
        String    svcMethods = null;
        String    optMethods = null;
        boolean   doEnq      = false;

        urnMap.put(name, svcUrn);

        for (Enumeration e = ddOptions.keys(); e.hasMoreElements(); )
        {
          String key = (String)e.nextElement();

          if      ("ServiceUrn".equals(key)) continue;
          else if ("enquire"   .equals(key)) doEnq = true;

          svcMethods = (svcMethods == null ? "" : svcMethods + " ") + (String)ddOptions.get(key);
          optMethods = (optMethods == null ? "" : optMethods + " ") + (String)ddOptions.get(key) + "=" + key;
        }

        File        fdd     = null;
        PrintWriter pwdd    = null;
        boolean     success = false;

        try
        {
          fdd  = new File(srcDir + File.separator + name + ddPostfix + ".xml");

          pwdd = cxt.getPrintWriter(fdd);

          pwdd.println("<?xml version='1.0'?>" + eol + eol +
                       "<!-- SOAP Deployment Descriptor for WSDL port '" + svcUrn + "', MQWF Process '" + name + "' -->" + eol +
                       "<!-- This file (\"" + fdd.getAbsolutePath() + "\") was generated -->" + eol +
                       "<!-- by %GeneratorName% on %GeneratedDate% -->" + eol +
                       "<!-- from FDL file \"%InputFileName%\" -->" + eol + eol +
                       "<isd:service xmlns:isd='http://xml.apache.org/xml-soap/deployment' id='urn:" + svcUrn + "'>" + eol +
                       " <isd:provider type   ='%ProviderClass%'" + eol +
                       "               scope  ='Application'" + eol +
                       "               methods='" + svcMethods + "'" + eol +
                       "               static ='false'>" + eol +
                       "  <!-- isd:java class='%ProviderClass%'/ -->" + eol +
                       "  <isd:option key='ServiceSchemaNamespace' value='%ServiceSchemaNamespace%'/>" + eol +
                       "  <isd:option key='ProcessTemplateName'    value='" + name + "'/>" + eol +
                       "  <isd:option key='MethodMap'              value='" + optMethods + "'/>");

          if (doProvider)
          {
            pwdd.println("  <isd:option key='ProviderMsgPipeClass'   value='%ProviderMsgPipeClass%'/>" + eol +
                         "  <isd:option key='ProviderXformerClass'   value='%ProviderXformerClass%'/>");
          }

          if (doAgent)
          {
            pwdd.println("  <isd:option key='AgentUserId'            value='%AgentUserId%'/>" + eol +
                         "  <isd:option key='AgentPassword'          value='%AgentPassword%'/>");
          }

          if (doJms)
          {
            pwdd.println("  <isd:option key='InitialContextFactory'  value='%InitialContextFactory%'/>" + eol +
                         "  <isd:option key='ProviderUrl'            value='%ProviderUrl%'/>" + eol +
                         "  <isd:option key='SecurityAuthentication' value='%SecurityAuthentication%'/>" + eol +
                         "  <isd:option key='QueueConnectionFactory' value='%QueueConnectionFactory%'/>" + eol +
                         "  <isd:option key='ReadQueue'              value='%ReadQueue%'/>" + eol +
                         "  <isd:option key='WriteQueue'             value='%WriteQueue%'/>");
          }

          if (doMQ)
          {
            pwdd.println("  <isd:option key='ReadQueueName'          value='%ReadQueueName%'/>" + eol +
                         "  <isd:option key='ReadQueueManagerName'   value='%ReadQueueManagerName%'/>" + eol +
                         "  <isd:option key='WriteQueueName'         value='%WriteQueueName%'/>" + eol +
                         "  <isd:option key='WriteQueueManagerName'  value='%WriteQueueManagerName%'/>");
          }

          pwdd.println(" </isd:provider>" + eol +
                       " <isd:faultListener>%FaultListener%</isd:faultListener>" + eol +
                       " <isd:mappings defaultRegistryClass='%ProviderRegistryClass%'>");

          List c = new ArrayList();

          if (input  != null) input .getClosure(c);
          if (output != null) output.getClosure(c);

          closureMap.put(name, c);

          for (int i = 0; i < c.size(); i++)
          {
            Structure s = (Structure)c.get(i);

            pwdd.println("  <isd:map encodingStyle     ='http://schemas.xmlsoap.org/soap/encoding/'" + eol +
                         "           xmlns:%SNS%         ='%ServiceSchemaNamespace%' qname='%SNS%:" + s.name + "'" + eol +
                         "           javaType          ='%SerializerPackage%." + s.name + "'" + eol +
                         "           xml2JavaClassName ='%SerializerPackage%." + s.name + "$Deserializer'" + eol +
                         "           java2XMLClassName ='%SerializerPackage%." + s.name + "$Deserializer'/>");
          }

          if (doEnq)
          {
            pwdd.println("  <isd:map encodingStyle     ='http://schemas.xmlsoap.org/soap/encoding/'" + eol +
                         "           xmlns:%WFNS%          ='%MqwfSchemaNamespace%' qname='%WFNS%:EnquiryResult'" + eol +
                         "           javaType          ='com.ibm.workflow.soap.provider.EnquiryResult'" + eol +
                         "           xml2JavaClassName ='com.ibm.workflow.soap.provider.EnquiryResult$Deserializer'" + eol +
                         "           java2XMLClassName ='com.ibm.workflow.soap.provider.EnquiryResult$Deserializer'/>");

            Structure er = new Structure(); er.name = "EnquiryResult"; c.add(er);
          }

          pwdd.println(" </isd:mappings>" + eol + "</isd:service>" + eol);

          success = true;
        }
        finally
        {
          if (pwdd != null) pwdd.close();

          if (!success && fdd != null) fdd.delete();
        }
      }

      private Structure input;
      private Structure output;
      private Hashtable ddOptions;
    }

    public void generateWSDL() throws ParseException, IOException
    {
      if (processes.size() > 0)
      {
        pw.println("<!-- ===================== -->" + eol +
                   "<!-- WSDL type definitions -->" + eol +
                   "<!-- ===================== -->" + eol +
                   eol +
                   "<wsdl:types>");

        if (doImport)
        {
          pw.println(" <wsdl:import namespace = '%ServiceSchemaNamespace%'" + eol +
                     "              location  = '%SchemaLocation%'/>");
        }
        else
        {
          pw.println("<xs:schema targetNamespace = '%ServiceSchemaNamespace%'" + eol +
                     "           xmlns:%SNS%       = '%ServiceSchemaNamespace%'" + eol +
                     "           xmlns:%WFNS%        = '%ServiceSchemaNamespace%'" + eol +
                     "           xmlns:xs        = '%XMLSchema%/XMLSchema'>" + eol + eol +
                     "<xs:simpleType name='STRING'>" + eol +
                     " <xs:restriction base='xs:string'>" + eol +
                     "  <xs:maxLength value='4194304'/>" + eol +
                     " </xs:restriction>" + eol +
                     "</xs:simpleType>" + eol + eol +
                     "<xs:simpleType name='LONG'>" + eol +
                     " <xs:restriction base='xs:integer'>" + eol +
                     "  <xs:minInclusive value='-2147483648'/>" + eol +
                     "  <xs:maxInclusive value='2147483647'/>" + eol +
                     " </xs:restriction>" + eol +
                     "</xs:simpleType>" + eol + eol +
                     "<xs:simpleType name='BINARY'>" + eol +
                     " <xs:restriction base='xs:base64Binary'>" + eol +
                     "  <xs:maxLength value='4194304'/>" + eol +
                     " </xs:restriction>" + eol +
                     "</xs:simpleType>" + eol + eol +
                     "<xs:simpleType name='FLOAT'>" + eol +
                     " <xs:restriction base='xs:double'>" + eol +
                     "  <xs:minInclusive value='-4.9E-324'/>" + eol +
                     "  <xs:maxInclusive value='1.7976931348623157E308'/>" + eol +
                     " </xs:restriction>" + eol +
                     "</xs:simpleType>" + eol + eol +
                     "<xs:simpleType name='STATE'>" + eol +
                     " <xs:restriction base='xs:string'>" + eol +
                     "  <xs:enumeration value='Deleted'/>" + eol +
                     "  <xs:enumeration value='Finished'/>" + eol +
                     "  <xs:enumeration value='Ready'/>" + eol +
                     "  <xs:enumeration value='Running'/>" + eol +
                     "  <xs:enumeration value='Suspended'/>" + eol +
                     "  <xs:enumeration value='Suspending'/>" + eol +
                     "  <xs:enumeration value='Terminated'/>" + eol +
                     "  <xs:enumeration value='Terminating'/>" + eol +
                     " </xs:restriction>" + eol +
                     "</xs:simpleType>" + eol + eol +
                     "<xs:complexType name='EnquiryResult'>" + eol +
                     " <xs:sequence>" + eol +
                     "  <xs:element name='FlowInstanceState'                type='%WFNS%:STATE'/>" + eol +
                     "  <xs:element name='FlowInstanceCreationTime'         type='xs:dateTime' minOccurs='0' maxOccurs='1'/>" + eol +
                     "  <xs:element name='FlowInstanceLastModificationTime' type='xs:dateTime' minOccurs='0' maxOccurs='1'/>" + eol +
                     " </xs:sequence>" + eol +
                     "</xs:complexType>" + eol);

          if (doNull)
          {
            pw.println("<xs:complexType name='nullType'/>" + eol);

            injectArray(pw, "LONG");
            injectArray(pw, "FLOAT");
            injectArray(pw, "STRING");
            injectArray(pw, "BINARY");
          }

          // Build ordered closure of STRUCTUREs for all processes, pass those to toXSDDefinition()

          List c = new ArrayList();

          for (Iterator it = processes.iterator(); it.hasNext(); )
          {
            Process p = (Process)it.next();

            if (p.input  != null) p.input .getClosure(c);
            if (p.output != null) p.output.getClosure(c);
          }

          for (int i = 0; i < c.size(); i++) ((Structure)c.get(i)).toXSDDefinition();

          pw.println("</xs:schema>");
        }

        pw.println("</wsdl:types>" + eol +
                   eol +
                   "<!-- ======================== -->" + eol +
                   "<!-- WSDL message definitions -->" + eol +
                   "<!-- ======================== -->" + eol);

        Hashtable done = new Hashtable();

        for (Iterator it = processes.iterator(); it.hasNext(); ) ((Process)it.next()).toWSDLMessage(done);

        pw.println("<!-- ========================= -->" + eol +
                   "<!-- WSDL portType definitions -->" + eol +
                   "<!-- ========================= -->" + eol);

        for (Iterator it = processes.iterator(); it.hasNext(); ) ((Process)it.next()).toWSDLPortType();

        pw.println("<!-- ======================== -->" + eol +
                   "<!-- WSDL binding definitions -->" + eol +
                   "<!-- ======================== -->" + eol);

        for (Iterator it = processes.iterator(); it.hasNext(); ) ((Process)it.next()).toWSDLBinding();

        pw.println("<!-- ======================== -->" + eol +
                   "<!-- WSDL service definitions -->" + eol +
                   "<!-- ======================== -->" + eol +
                   eol +
                   "<wsdl:service name='%ServiceName%'>" + eol);

        for (Iterator it = processes.iterator(); it.hasNext(); ) ((Process)it.next()).toWSDLService();

        pw.println("</wsdl:service>");

        for (Iterator it = processes.iterator(); it.hasNext(); ) ((Process)it.next()).toDeploymentDescriptor();
      }
      else System.out.println("No process definition found in FDL.");
    }

    /**
     *
     * Visit an FDL <a href="../../../../../../../fdlgrammar.html#prod9">Process</a>.
     *
     */

    public Object visit(FdlProcess node, Object data) throws ParseException
    {
      process = new Process(); // Open a new Process to get the process data

      node.childrenAccept(this, Boolean.TRUE); // Recurse into child nodes, data != null un-NOP's FdlName visits.

      processes.add(process); // Add the Process to the list of processes

      process = null;

      return data;
    }

    /**
     *
     * Visit an FDL <a href="../../../../../../../fdlgrammar.html#prod36">ArgumentList</a>.
     *
     */

    public Object visit(FdlArgumentList node, Object data) throws ParseException
    {
      node.childrenAccept(this, data);

      return data;
    }

    /**
     *
     * Visit an FDL <a href="../../../../../../../fdlgrammar.html#prod28">Name</a>.
     *
     */

    public Object visit(FdlName node, Object data) throws ParseException
    {
      if (process == null) return super.visit(node, data);

      String s = removeBlanks(node);

      if (process.name == null)
      {
        process.name = s;
      }
      else if (process.input == null)
      {
        for (int i = 0; i < structures.size(); i++)
        {
          if (((Structure)structures.get(i)).name.equals(s)) process.input = (Structure)structures.get(i);
        }

        if (process.input == null) throw new ParseException("Definition of STRUCTURE '" + s + "' not found.");
      }
      else if (process.output == null)
      {
        for (int i = 0; i < structures.size(); i++)
        {
          if (((Structure)structures.get(i)).name.equals(s)) process.output = (Structure)structures.get(i);
        }

        if (process.output == null) throw new ParseException("Definition of STRUCTURE '" + s + "' not found.");
      }

      return data;
    }

    /**
     *
     * Construct a Visitor, passing Make context to the base visitor. Also, cache some configuration information.
     *
     */

    public Visitor(Make cxt)
    {
      super(cxt);

    }

    // Helper Functions
    // ----------------

    private void injectArray(PrintWriter pw, String type)
    {
      pw.println("<xs:complexType name='" + type + "_ARRAY' mixed='true'>" + eol +
                 " <xs:sequence>" + eol +
                 "  <xs:element name='null' type='%WFNS%:nullType'/>" + eol +
                 " </xs:sequence>" + eol +
                 "</xs:complexType>" + eol);
    }

    // Members
    // -------

    private Process process   = null;
    private List    processes = new ArrayList(10);
  }

  protected void postGenerate(String tgt) throws Exception
  {
    Utils.initXslStreamHandler(this);

    Hashtable ht = (Hashtable)getConfig().get("SUBST");

    mqwfPrefix = (String)ht.get("WFNS");

    Document     doc = Utils.parseWithImports(tgt);
    DOMSource    ds  = new DOMSource(doc);
    StreamResult sr  = new StreamResult(System.out);

    transform(ds, sr, "xsl:./wsdl2class.xsl");
    transform(ds, sr, "xsl:./wsdl2proxy.xsl");
    transform(ds, sr, "xsl:./wsdl2ant.xsl");

    if (location != null)
    {
      new File(tgt).delete();

      int    pos  = tgt.lastIndexOf('.');
      String intf = (pos <= 0 ? tgt : tgt.substring(0, pos)) + ifPostfix + ".wsdl";
      Attr   attr = ((Element)doc.getElementsByTagNameNS(Utils.NS_URI_WSDL(), "definitions").item(0)).getAttributeNode("xmlns:tns");

      attr.setValue(attr.getValue() + ifPostfix);
      ht.put("OutputFileName", intf);

      transform(ds, new StreamResult(new FileWriter(intf)), "xsl:./wsdl2if.xsl");

      ht.put("OutputFileName", tgt);

      transform(ds, new StreamResult(new FileWriter(tgt)), "xsl:./wsdl2impl.xsl");
    }
  }

  private final void transform(Source src, Result out, String ss)
    throws TransformerConfigurationException, TransformerException
  {
    Hashtable   h = (Hashtable)getConfig().get("SUBST");
    Transformer t = tFactory.newTransformer(new StreamSource(Utils.getXslStream(ss)));

    t.setParameter("generator", name());
    t.setParameter("ppackage",  h.get("ProxyPackage"));
    t.setParameter("spackage",  h.get("SerializerPackage"));
    t.setParameter("basedir",   srcDir);
    t.setParameter("output",    h.get("OutputFileName"));
    t.setParameter("ddpostfix", ddPostfix);
    t.setParameter("sns",       h.get("SNS") + ":");
    t.setParameter("sil",       location + "/");

    t.transform(src, out);
  }

  private WSDL(String[] args) throws Exception { super(args); }

  public static void main(String args[]) throws Exception
  {
    WSDL wsdl = new WSDL(args);

    Hashtable ht = (Hashtable)wsdl.getConfig().get("SUBST");

    proxyNS  = (String)ht.get("ServiceNamespace");
    schemaNS = (String)ht.get("ServiceSchemaNamespace");
    mqwfNS   = (String)ht.get("MqwfSchemaNamespace");
    location = (String)ht.get("ServiceImportLocation");

    ht.put("SerializerPackage", Utils.ns2pkg(schemaNS));
    ht.put("ProxyPackage",      Utils.ns2pkg(proxyNS));

    String providerClass = (String)ht.get("ProviderClass");

    if ("com.ibm.workflow.soap.provider.Provider".equals(providerClass)) doProvider = true;

    String pipeClass = (String)ht.get("ProviderMsgPipeClass");

    if ("com.ibm.workflow.soap.provider.MQPipe" .equals(pipeClass)) doJms = false; // Not open/closed (JRNXI)
    if ("com.ibm.workflow.soap.provider.JmsPipe".equals(pipeClass)) doMQ  = false; // Not open/closed (JRNXI)

    if (ht.get("AgentUserId") == null || ht.get("AgentPassword") == null) doAgent = false;

    String svcName = (String)ht.get("ServiceName");

    ht = (Hashtable)wsdl.getConfig().get("OPTIONS");

    String ddpfx = (String)ht.get("DDPostfix");

    ddPostfix = ddpfx == null ? "Deployment" : ddpfx;
//  doImport  = "yes".equals((String)ht.get("ImportSchema")); // (JRNYI)

    String dir = (String)ht.get("SourceDirectory");

    if (dir != null) srcDir = dir;

    File tmp = new File(srcDir);

    if (!tmp.exists()) tmp.mkdirs();

    String tgt = args.length > 1 ? args[1] : svcName + "." + wsdl.ext();

    if (!new File(tgt).isAbsolute()) tgt = srcDir + File.separator + tgt;

    wsdl.make(tgt);
  }

  public static final String getUrn(NodeList type) { return (String)urnMap.get(type.item(0).getNodeValue()); }

  public static final NodeSet getComplex(NodeList nl)
  {
    Map       cts = new HashMap();
    NodeSet   ct  = new NodeSet();

    for (int i = 0; i < nl.getLength(); i++)
    {
      Element e = (Element)nl.item(i);
      String  s = e.getAttribute("type");
      String  c = e.getAttribute("maxOccurs");

      if (s == null || s.startsWith("xs:") || s.startsWith(mqwfPrefix)) continue;
      if (c == null || c.length() == 0 || Integer.parseInt(c) > 1) continue;

      cts.put(stripNS(s), new Boolean(true));
    }

    if (cts.size() > 0)
    {
      Document owner = nl.item(0).getOwnerDocument();

      for (Iterator it = cts.keySet().iterator(); it.hasNext(); ) ct.addNode(owner.createTextNode((String)it.next()));
    }

    return ct;
  }

  public static final NodeSet getClosure(NodeList name)
  {
    List    c  = (List)closureMap.get(name.item(0).getNodeValue());
    NodeSet cl = new NodeSet();

    if (c != null && c.size() > 0)
    {
      Document owner = name.item(0).getOwnerDocument();

      for (Iterator it = c.iterator(); it.hasNext(); )
      {
        Element e = owner.createElement("STRUCTURE");
        String  s = ((Visitor.Structure)it.next()).name;

        e.setAttribute("name", s);
        e.setAttribute("namespace", "EnquiryResult".equals(s) ? mqwfNS : schemaNS);

        cl.addNode(e);
      }
    }

    return cl;
  }

  public static final String globalName(NodeList nl) { return stripNS(nl.item(0).getNodeValue()); }

  public static final String getJavaType(String pkg, NodeList type, NodeList card)
  {
    if (type.getLength() == 0) return "void";

    String s = type.item(0).getNodeValue();
    String c = card == null || card.getLength() == 0 ? "1" : card.item(0).getNodeValue();
    String j = stripNS(s);

    if (s.startsWith("xs:") || s.startsWith(mqwfPrefix)) s = j.toLowerCase();

    String a = s.endsWith("_array") || Integer.parseInt(c) > 1 ? "[]" : "";

    if      (s.startsWith("long"  )) j = "Long";
    else if (s.startsWith("float" )) j = "Double";
    else if (s.startsWith("double")) j = "Double";
    else if (s.startsWith("binary")) j = "byte[]";
    else if (s.startsWith("string")) j = "String";
    else if (pkg.length() > 0)       j = pkg + "." + j;

    return j + a;
  }

  public static final String getSerializer(NodeList type, NodeList card)
  {
    String s = type.item(0).getNodeValue();
    String c = card == null || card.getLength() == 0 ? "1" : card.item(0).getNodeValue();
    String j = stripNS(s);

    if (s.startsWith("xs:") || s.startsWith(mqwfPrefix)) s = j.toLowerCase();

    if (s.endsWith("_array") || Integer.parseInt(c) > 1) return "array";

    if      (s.equals("long"  )) return "long";
    else if (s.equals("float" )) return "double";
    else if (s.equals("double")) return "double";
    else if (s.equals("binary")) return "binary";
    else if (s.equals("string")) return "string";
    else                         return j;
  }

  public static final String getQName(String pkg, NodeList type)
  {
    String s = type.item(0).getNodeValue();
    String j = null;
    String n = "Constants.NS_URI_2001_SCHEMA_XSD"; // parametrize (JRNYI)

    if (s.startsWith("xs:") || s.startsWith(mqwfPrefix)) s = stripNS(s).toLowerCase();

    if      (s.equals("long_array"  )) j = "long";
    else if (s.equals("float_array" )) j = "double";
    else if (s.equals("binary_array")) j = "byte[]";
    else if (s.equals("string_array")) j = "string";
    else                               { j = stripNS(s); n = "\"\""; }

    return "new QName(" + n + ", \"" + j + "\")";
  }

  public static final String getFileName(String type, String dir, String pkg, String postfix)
  {
    if (pkg != null && !pkg.equals("")) dir = dir + File.separator + pkg.replace('.', File.separatorChar);

    return dir + File.separatorChar + type + postfix;
  }

  private static final String stripNS(String s) { return s.substring(1 + s.indexOf(':')); }

  private static String  srcDir     = ".";
  private static String  mqwfPrefix = null;
  private static String  proxyNS    = null;
  private static String  schemaNS   = null;
  private static String  mqwfNS     = null;
  private static String  location   = null;
  private static String  ddPostfix  = null;
  private static boolean doImport   = false;
  private static boolean doProvider = false;
  private static boolean doAgent    = true;
  private static boolean doJms      = true;
  private static boolean doMQ       = true;

  private static final Hashtable          urnMap     = new Hashtable();
  private static final Hashtable          closureMap = new Hashtable();
  private static final TransformerFactory tFactory   = TransformerFactory.newInstance();

  protected final Class  visitor() { return Visitor.class; }
  protected final String ext    () { return "wsdl"; }
  protected final String tag    () { return "<!-- _GENERATED_WSDL_ -->"; }
  protected final String name   () { return "MQWF WSDL Generator V1.2"; }
  protected final String usage  () { return "<FDL input file> [WSDL output file, [Configuration file]]"; }

  private static final String ifPostfix = "-interface";
}
